
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

// http://www.apache.org/licenses/LICENSE-2.0

// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// This file was initially generated by gen_to_start.go (add link), as a start
// of the Traffic Ops golang data model

package api

import (
	"encoding/json"
	_ "github.com/apache/trafficcontrol/traffic_ops/experimental/server/output_format" // needed for swagger
	"github.com/jmoiron/sqlx"
	"log"
	"time"
)

type Asns struct {
	Asn       int64     `db:"asn" json:"asn"`
	CreatedAt time.Time `db:"created_at" json:"createdAt"`
	Links     AsnsLinks `json:"_links" db:-`
}

type AsnsLinks struct {
	Self            string          `db:"self" json:"_self"`
	CachegroupsLink CachegroupsLink `json:"cachegroups" db:-`
}

// @Title getAsnsById
// @Description retrieves the asns information for a certain id
// @Accept  application/json
// @Param   id              path    int     false        "The row id"
// @Success 200 {array}    Asns
// @Resource /api/2.0
// @Router /api/2.0/asns/{id} [get]
func getAsn(asn int64, db *sqlx.DB) (interface{}, error) {
	ret := []Asns{}
	arg := Asns{}
	arg.Asn = asn
	queryStr := "select *, concat('" + API_PATH + "asns/', asn) as self"
	queryStr += ", concat('" + API_PATH + "cachegroups/', cachegroups) as cachegroups_name_ref"
	queryStr += " from asns WHERE asn=:asn"
	nstmt, err := db.PrepareNamed(queryStr)
	err = nstmt.Select(&ret, arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	nstmt.Close()
	return ret, nil
}

// @Title getAsnss
// @Description retrieves the asns
// @Accept  application/json
// @Success 200 {array}    Asns
// @Resource /api/2.0
// @Router /api/2.0/asns [get]
func getAsns(db *sqlx.DB) (interface{}, error) {
	ret := []Asns{}
	queryStr := "select *, concat('" + API_PATH + "asns/', asn) as self"
	queryStr += ", concat('" + API_PATH + "cachegroups/', cachegroups) as cachegroups_name_ref"
	queryStr += " from asns"
	err := db.Select(&ret, queryStr)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return ret, nil
}

// @Title postAsns
// @Description enter a new asns
// @Accept  application/json
// @Param                 Body body     Asns   true "Asns object that should be added to the table"
// @Success 200 {object}    output_format.ApiWrapper
// @Resource /api/2.0
// @Router /api/2.0/asns [post]
func postAsn(payload []byte, db *sqlx.DB) (interface{}, error) {
	var v Asns
	err := json.Unmarshal(payload, &v)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	sqlString := "INSERT INTO asns("
	sqlString += "asn"
	sqlString += ",cachegroups"
	sqlString += ",created_at"
	sqlString += ") VALUES ("
	sqlString += ":asn"
	sqlString += ",:cachegroups"
	sqlString += ",:created_at"
	sqlString += ")"
	result, err := db.NamedExec(sqlString, v)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}

// @Title putAsns
// @Description modify an existing asnsentry
// @Accept  application/json
// @Param   id              path    int     true        "The row id"
// @Param                 Body body     Asns   true "Asns object that should be added to the table"
// @Success 200 {object}    output_format.ApiWrapper
// @Resource /api/2.0
// @Router /api/2.0/asns/{id}  [put]
func putAsn(asn int64, payload []byte, db *sqlx.DB) (interface{}, error) {
	var arg Asns
	err := json.Unmarshal(payload, &arg)
	arg.Asn = asn
	if err != nil {
		log.Println(err)
		return nil, err
	}
	sqlString := "UPDATE asns SET "
	sqlString += "asn = :asn"
	sqlString += ",cachegroups = :cachegroups"
	sqlString += ",created_at = :created_at"
	sqlString += " WHERE asn=:asn"
	result, err := db.NamedExec(sqlString, arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}

// @Title delAsnsById
// @Description deletes asns information for a certain id
// @Accept  application/json
// @Param   id              path    int     false        "The row id"
// @Success 200 {array}    Asns
// @Resource /api/2.0
// @Router /api/2.0/asns/{id} [delete]
func delAsn(asn int64, db *sqlx.DB) (interface{}, error) {
	arg := Asns{}
	arg.Asn = asn
	result, err := db.NamedExec("DELETE FROM asns WHERE asn=:asn", arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}
