
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

// http://www.apache.org/licenses/LICENSE-2.0

// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// This file was initially generated by gen_to_start.go (add link), as a start
// of the Traffic Ops golang data model

package api

import (
	"encoding/json"
	_ "github.com/apache/trafficcontrol/traffic_ops/experimental/server/output_format" // needed for swagger
	"github.com/jmoiron/sqlx"
	null "gopkg.in/guregu/null.v3"
	"log"
	"time"
)

type Cachegroups struct {
	Name        string           `db:"name" json:"name"`
	Description string           `db:"description" json:"description"`
	Latitude    null.Int         `db:"latitude" json:"latitude"`
	Longitude   null.Int         `db:"longitude" json:"longitude"`
	CreatedAt   time.Time        `db:"created_at" json:"createdAt"`
	Links       CachegroupsLinks `json:"_links" db:-`
}

type CachegroupsLinks struct {
	Self                 string               `db:"self" json:"_self"`
	CachegroupsLink      CachegroupsLink      `json:"cachegroups" db:-`
	CachegroupsTypesLink CachegroupsTypesLink `json:"cachegroups_types" db:-`
}

type CachegroupsLink struct {
	ID  string `db:"cachegroup" json:"name"`
	Ref string `db:"cachegroups_name_ref" json:"_ref"`
}

// @Title getCachegroupsById
// @Description retrieves the cachegroups information for a certain id
// @Accept  application/json
// @Param   id              path    int     false        "The row id"
// @Success 200 {array}    Cachegroups
// @Resource /api/2.0
// @Router /api/2.0/cachegroups/{id} [get]
func getCachegroup(name string, db *sqlx.DB) (interface{}, error) {
	ret := []Cachegroups{}
	arg := Cachegroups{}
	arg.Name = name
	queryStr := "select *, concat('" + API_PATH + "cachegroups/', name) as self"
	queryStr += ", concat('" + API_PATH + "cachegroups/', parent_cachegroup) as cachegroups_name_ref"
	queryStr += ", concat('" + API_PATH + "cachegroups_types/', type) as cachegroups_types_name_ref"
	queryStr += " from cachegroups WHERE name=:name"
	nstmt, err := db.PrepareNamed(queryStr)
	err = nstmt.Select(&ret, arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	nstmt.Close()
	return ret, nil
}

// @Title getCachegroupss
// @Description retrieves the cachegroups
// @Accept  application/json
// @Success 200 {array}    Cachegroups
// @Resource /api/2.0
// @Router /api/2.0/cachegroups [get]
func getCachegroups(db *sqlx.DB) (interface{}, error) {
	ret := []Cachegroups{}
	queryStr := "select *, concat('" + API_PATH + "cachegroups/', name) as self"
	queryStr += ", concat('" + API_PATH + "cachegroups/', parent_cachegroup) as cachegroups_name_ref"
	queryStr += ", concat('" + API_PATH + "cachegroups_types/', type) as cachegroups_types_name_ref"
	queryStr += " from cachegroups"
	err := db.Select(&ret, queryStr)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return ret, nil
}

// @Title postCachegroups
// @Description enter a new cachegroups
// @Accept  application/json
// @Param                 Body body     Cachegroups   true "Cachegroups object that should be added to the table"
// @Success 200 {object}    output_format.ApiWrapper
// @Resource /api/2.0
// @Router /api/2.0/cachegroups [post]
func postCachegroup(payload []byte, db *sqlx.DB) (interface{}, error) {
	var v Cachegroups
	err := json.Unmarshal(payload, &v)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	sqlString := "INSERT INTO cachegroups("
	sqlString += "name"
	sqlString += ",description"
	sqlString += ",latitude"
	sqlString += ",longitude"
	sqlString += ",parent_cachegroup"
	sqlString += ",type"
	sqlString += ",created_at"
	sqlString += ") VALUES ("
	sqlString += ":name"
	sqlString += ",:description"
	sqlString += ",:latitude"
	sqlString += ",:longitude"
	sqlString += ",:parent_cachegroup"
	sqlString += ",:type"
	sqlString += ",:created_at"
	sqlString += ")"
	result, err := db.NamedExec(sqlString, v)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}

// @Title putCachegroups
// @Description modify an existing cachegroupsentry
// @Accept  application/json
// @Param   id              path    int     true        "The row id"
// @Param                 Body body     Cachegroups   true "Cachegroups object that should be added to the table"
// @Success 200 {object}    output_format.ApiWrapper
// @Resource /api/2.0
// @Router /api/2.0/cachegroups/{id}  [put]
func putCachegroup(name string, payload []byte, db *sqlx.DB) (interface{}, error) {
	var arg Cachegroups
	err := json.Unmarshal(payload, &arg)
	arg.Name = name
	if err != nil {
		log.Println(err)
		return nil, err
	}
	sqlString := "UPDATE cachegroups SET "
	sqlString += "name = :name"
	sqlString += ",description = :description"
	sqlString += ",latitude = :latitude"
	sqlString += ",longitude = :longitude"
	sqlString += ",parent_cachegroup = :parent_cachegroup"
	sqlString += ",type = :type"
	sqlString += ",created_at = :created_at"
	sqlString += " WHERE name=:name"
	result, err := db.NamedExec(sqlString, arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}

// @Title delCachegroupsById
// @Description deletes cachegroups information for a certain id
// @Accept  application/json
// @Param   id              path    int     false        "The row id"
// @Success 200 {array}    Cachegroups
// @Resource /api/2.0
// @Router /api/2.0/cachegroups/{id} [delete]
func delCachegroup(name string, db *sqlx.DB) (interface{}, error) {
	arg := Cachegroups{}
	arg.Name = name
	result, err := db.NamedExec("DELETE FROM cachegroups WHERE name=:name", arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}
