
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

// http://www.apache.org/licenses/LICENSE-2.0

// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// This file was initially generated by gen_to_start.go (add link), as a start
// of the Traffic Ops golang data model

package api

import (
	"encoding/json"
	_ "github.com/apache/trafficcontrol/traffic_ops/experimental/server/output_format" // needed for swagger
	"github.com/jmoiron/sqlx"
	"log"
	"time"
)

type Cdns struct {
	Name      string    `db:"name" json:"name"`
	CreatedAt time.Time `db:"created_at" json:"createdAt"`
	Links     CdnsLinks `json:"_links" db:-`
}

type CdnsLinks struct {
	Self string `db:"self" json:"_self"`
}

type CdnsLink struct {
	ID  string `db:"cdn" json:"name"`
	Ref string `db:"cdns_name_ref" json:"_ref"`
}

// @Title getCdnsById
// @Description retrieves the cdns information for a certain id
// @Accept  application/json
// @Param   id              path    int     false        "The row id"
// @Success 200 {array}    Cdns
// @Resource /api/2.0
// @Router /api/2.0/cdns/{id} [get]
func getCdn(name string, db *sqlx.DB) (interface{}, error) {
	ret := []Cdns{}
	arg := Cdns{}
	arg.Name = name
	queryStr := "select *, concat('" + API_PATH + "cdns/', name) as self"
	queryStr += " from cdns WHERE name=:name"
	nstmt, err := db.PrepareNamed(queryStr)
	err = nstmt.Select(&ret, arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	nstmt.Close()
	return ret, nil
}

// @Title getCdnss
// @Description retrieves the cdns
// @Accept  application/json
// @Success 200 {array}    Cdns
// @Resource /api/2.0
// @Router /api/2.0/cdns [get]
func getCdns(db *sqlx.DB) (interface{}, error) {
	ret := []Cdns{}
	queryStr := "select *, concat('" + API_PATH + "cdns/', name) as self"
	queryStr += " from cdns"
	err := db.Select(&ret, queryStr)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return ret, nil
}

// @Title postCdns
// @Description enter a new cdns
// @Accept  application/json
// @Param                 Body body     Cdns   true "Cdns object that should be added to the table"
// @Success 200 {object}    output_format.ApiWrapper
// @Resource /api/2.0
// @Router /api/2.0/cdns [post]
func postCdn(payload []byte, db *sqlx.DB) (interface{}, error) {
	var v Cdns
	err := json.Unmarshal(payload, &v)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	sqlString := "INSERT INTO cdns("
	sqlString += "name"
	sqlString += ",created_at"
	sqlString += ") VALUES ("
	sqlString += ":name"
	sqlString += ",:created_at"
	sqlString += ")"
	result, err := db.NamedExec(sqlString, v)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}

// @Title putCdns
// @Description modify an existing cdnsentry
// @Accept  application/json
// @Param   id              path    int     true        "The row id"
// @Param                 Body body     Cdns   true "Cdns object that should be added to the table"
// @Success 200 {object}    output_format.ApiWrapper
// @Resource /api/2.0
// @Router /api/2.0/cdns/{id}  [put]
func putCdn(name string, payload []byte, db *sqlx.DB) (interface{}, error) {
	var arg Cdns
	err := json.Unmarshal(payload, &arg)
	arg.Name = name
	if err != nil {
		log.Println(err)
		return nil, err
	}
	sqlString := "UPDATE cdns SET "
	sqlString += "name = :name"
	sqlString += ",created_at = :created_at"
	sqlString += " WHERE name=:name"
	result, err := db.NamedExec(sqlString, arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}

// @Title delCdnsById
// @Description deletes cdns information for a certain id
// @Accept  application/json
// @Param   id              path    int     false        "The row id"
// @Success 200 {array}    Cdns
// @Resource /api/2.0
// @Router /api/2.0/cdns/{id} [delete]
func delCdn(name string, db *sqlx.DB) (interface{}, error) {
	arg := Cdns{}
	arg.Name = name
	result, err := db.NamedExec("DELETE FROM cdns WHERE name=:name", arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}
