
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

// http://www.apache.org/licenses/LICENSE-2.0

// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

// This file was initially generated by gen_to_start.go (add link), as a start
// of the Traffic Ops golang data model

package api

import (
	"encoding/json"
	_ "github.com/apache/trafficcontrol/traffic_ops/experimental/server/output_format" // needed for swagger
	"github.com/jmoiron/sqlx"
	"log"
	"time"
)

type DeliveryservicesServers struct {
	Deliveryservice string                       `db:"deliveryservice" json:"deliveryservice"`
	Server          string                       `db:"server" json:"server"`
	CreatedAt       time.Time                    `db:"created_at" json:"createdAt"`
	Links           DeliveryservicesServersLinks `json:"_links" db:-`
}

type DeliveryservicesServersLinks struct {
	Self string `db:"self" json:"_self"`
}

// @Title getDeliveryservicesServersById
// @Description retrieves the deliveryservices_servers information for a certain id
// @Accept  application/json
// @Param   id              path    int     false        "The row id"
// @Success 200 {array}    DeliveryservicesServers
// @Resource /api/2.0
// @Router /api/2.0/deliveryservices_servers/{id} [get]
func getDeliveryservicesServer(deliveryservice string, server string, db *sqlx.DB) (interface{}, error) {
	ret := []DeliveryservicesServers{}
	arg := DeliveryservicesServers{}
	arg.Deliveryservice = deliveryservice
	arg.Server = server
	queryStr := "select *, concat('" + API_PATH + "deliveryservices_servers', '/deliveryservice/', deliveryservice, '/server/', server) as self"
	queryStr += " from deliveryservices_servers WHERE deliveryservice=:deliveryservice AND server=:server"
	nstmt, err := db.PrepareNamed(queryStr)
	err = nstmt.Select(&ret, arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	nstmt.Close()
	return ret, nil
}

// @Title getDeliveryservicesServerss
// @Description retrieves the deliveryservices_servers
// @Accept  application/json
// @Success 200 {array}    DeliveryservicesServers
// @Resource /api/2.0
// @Router /api/2.0/deliveryservices_servers [get]
func getDeliveryservicesServers(db *sqlx.DB) (interface{}, error) {
	ret := []DeliveryservicesServers{}
	queryStr := "select *, concat('" + API_PATH + "deliveryservices_servers', '/deliveryservice/', deliveryservice, '/server/', server) as self"
	queryStr += " from deliveryservices_servers"
	err := db.Select(&ret, queryStr)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return ret, nil
}

// @Title postDeliveryservicesServers
// @Description enter a new deliveryservices_servers
// @Accept  application/json
// @Param                 Body body     DeliveryservicesServers   true "DeliveryservicesServers object that should be added to the table"
// @Success 200 {object}    output_format.ApiWrapper
// @Resource /api/2.0
// @Router /api/2.0/deliveryservices_servers [post]
func postDeliveryservicesServer(payload []byte, db *sqlx.DB) (interface{}, error) {
	var v DeliveryservicesServers
	err := json.Unmarshal(payload, &v)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	sqlString := "INSERT INTO deliveryservices_servers("
	sqlString += "deliveryservice"
	sqlString += ",server"
	sqlString += ",created_at"
	sqlString += ") VALUES ("
	sqlString += ":deliveryservice"
	sqlString += ",:server"
	sqlString += ",:created_at"
	sqlString += ")"
	result, err := db.NamedExec(sqlString, v)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}

// @Title putDeliveryservicesServers
// @Description modify an existing deliveryservices_serversentry
// @Accept  application/json
// @Param   id              path    int     true        "The row id"
// @Param                 Body body     DeliveryservicesServers   true "DeliveryservicesServers object that should be added to the table"
// @Success 200 {object}    output_format.ApiWrapper
// @Resource /api/2.0
// @Router /api/2.0/deliveryservices_servers/{id}  [put]
func putDeliveryservicesServer(deliveryservice string, server string, payload []byte, db *sqlx.DB) (interface{}, error) {
	var arg DeliveryservicesServers
	err := json.Unmarshal(payload, &arg)
	arg.Deliveryservice = deliveryservice
	arg.Server = server
	if err != nil {
		log.Println(err)
		return nil, err
	}
	sqlString := "UPDATE deliveryservices_servers SET "
	sqlString += "deliveryservice = :deliveryservice"
	sqlString += ",server = :server"
	sqlString += ",created_at = :created_at"
	sqlString += " WHERE deliveryservice=:deliveryservice AND server=:server"
	result, err := db.NamedExec(sqlString, arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}

// @Title delDeliveryservicesServersById
// @Description deletes deliveryservices_servers information for a certain id
// @Accept  application/json
// @Param   id              path    int     false        "The row id"
// @Success 200 {array}    DeliveryservicesServers
// @Resource /api/2.0
// @Router /api/2.0/deliveryservices_servers/{id} [delete]
func delDeliveryservicesServer(deliveryservice string, server string, db *sqlx.DB) (interface{}, error) {
	arg := DeliveryservicesServers{}
	arg.Deliveryservice = deliveryservice
	arg.Server = server
	result, err := db.NamedExec("DELETE FROM deliveryservices_servers WHERE deliveryservice=:deliveryservice AND server=:server", arg)
	if err != nil {
		log.Println(err)
		return nil, err
	}
	return result, err
}
