/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */

var EndpointService = function(Restangular, locationUtils, messageModel) {

	this.getEndpoints = function(queryParams) {
		return Restangular.all('api_capabilities').getList(queryParams);
	};

	this.getEndpoint = function(id) {
		return Restangular.one("api_capabilities", id).get();
	};

	this.createEndpoint = function(endpoint) {
		return Restangular.service('api_capabilities').post(endpoint)
			.then(
				function() {
					messageModel.setMessages([ { level: 'success', text: 'Endpoint created' } ], true);
					locationUtils.navigateToPath('/endpoints');
				},
				function(fault) {
					messageModel.setMessages(fault.data.alerts, false);
				}
			);
	};

	this.updateEndpoint = function(endpoint) {
		return endpoint.put()
			.then(
				function() {
					messageModel.setMessages([ { level: 'success', text: 'Endpoint updated' } ], false);
				},
				function(fault) {
					messageModel.setMessages(fault.data.alerts, false);
				}
			);
	};

	this.deleteEndpoint = function(id) {
		return Restangular.one("api_capabilities", id).remove()
			.then(
				function() {
					messageModel.setMessages([ { level: 'success', text: 'Endpoint deleted' } ], true);
				},
				function(fault) {
					messageModel.setMessages(fault.data.alerts, true);
				}
			);
	};


};

EndpointService.$inject = ['Restangular', 'locationUtils', 'messageModel'];
module.exports = EndpointService;
